package PIANOS.datastructures;

import PIANOS.exceptions.*;
import PIANOS.datastructures.*;

import java.util.*;

/*
Note: The thing calling generaterFreqCode and generateGenCode
must add _dp to real constants. For example, "1.0_dp", not "1.0".
*/

public class BetaDistribution extends Distribution
{

    public BetaDistribution()
    {
        super(2);
        parameterType[0] = 1; // real
        parameterType[1] = 1; // real
    }

    public ArrayList<String>  getIntroduction()
	{
		ArrayList<String> result = new ArrayList<String>();
		result.add("EXTERNAL G01EEF");
        result.add("EXTERNAL G05FEF");
        return result;
	}

	public ArrayList<String> getFreqCode(String[] parameters) throws IllegalParametersException
	{
		// Parameters are: a, b, point, result

		if (parameters.length != 4)
		{
			throw new IllegalParametersException("Trying to use beta distribution with illegal parameters.");
		}

		/*
		The code we want to generate looks like this:
        ifail = -1
		CALL G01EEF(point, a, b, 1.0_dp, temp_real, temp_real, result, ifail)
		IF (ifail /= 0) THEN
            WRITE (*, *) 'Error when trying to calculate frequency of the beta distribution, parameters: a = ', a, 'b = ', b
            STOP
        END IF
        */

		ArrayList<String> result = new ArrayList<String>();

		result.add("ifail = -1");
		result.add("CALL G01EEF(" + parameters[2] +", " + parameters[0] + ", " + parameters[1] +
            ", 1.0_dp, temp_real, temp_real, " + parameters[3] + ", ifail)");
		result.add("IF (ifail /= 0) THEN");
		result.add("WRITE (*, *) 'Error when trying to calculate frequency of the beta distribution, parameters: a = ', " +
            parameters[0] + ", 'b = ', " + parameters[1]);
		result.add("STOP");
        result.add("END IF");

		return result;
	}

	public ArrayList<String> getGenCode(String[] parameters) throws IllegalParametersException
	{
        // Parameters are: a, b, result
        if (parameters.length != 3)
		{
			throw new IllegalParametersException("Trying to use beta distribution with illegal parameters.");
		}

        /*
        The code we want to generate looks like this:
        ifail = 0
        CALL G05FEF(a, b, SIZE(result), result, ifail)
        IF (ifail /= 0) THEN
            WRITE (*, *) 'Error when trying to generate random numbers from beta distribution, parameters: a = ', a, 'b = ', b
            STOP
        END IF
        */

        ArrayList<String> result = new ArrayList<String>();

        result.add("ifail = 0");
        result.add("CALL G05FEF(" + parameters[0] + ", " + parameters[1] + ", SIZE(" + parameters[2] +
            "), " + parameters[2] + ", ifail)");
        result.add("IF (ifail /= 0) THEN");
        result.add("WRITE (*, *) 'Error when trying to generate random numbers from beta distribution, " +
            "parameters: a = ', " + parameters[0] + ", 'b = ', " + parameters[1]);
        result.add("STOP");
        result.add("END IF");

        return result;

        /*
        Note:
        The code generated by this method uses the following already introduced variables:
        INTEGER :: ifail
        */
	}
}