package PIANOS.datastructures;

import PIANOS.exceptions.*;
import PIANOS.datastructures.*;

import java.util.*;

/*
Note: The thing calling generaterFreqCode and generateGenCode
must add _dp to real constants. For example, "1.0_dp", not "1.0".
*/

public class DiscreteUniformDistribution extends Distribution
{

    public DiscreteUniformDistribution()
    {
        super(2);
        parameterType[0] = 0; // integer
        parameterType[1] = 0; // integer
    }

    public ArrayList<String> getIntroduction()
	{
		ArrayList<String> result = new ArrayList<String>();
		result.add("INTEGER G05DYF");
        result.add("EXTERNAL G05DYF");
        return result;
	}

	public ArrayList<String> getFreqCode(String[] parameters) throws IllegalParametersException
	{
		// This is not done with NAG because it's a trivial calculation
		// Parameters are: m, n, point, result

		if (parameters.length != 4)
		{
			throw new IllegalParametersException("Trying to use discrete uniform distribution with illegal parameters.");
		}

		/*
		The code we want to generate looks like this:
		IF (point < m .OR. point > n) THEN
			result = 0
		ELSE
			result = 1 / (n - (m) +1)
		END IF
		*/

		ArrayList<String> result = new ArrayList<String>();

		result.add("IF (" + parameters[2] + " < " + parameters[0] + " .OR. " +
			parameters[2] + " > " + parameters[1] + ") THEN");
		result.add(parameters[3] + " = 0");
		result.add("ELSE");
		result.add(parameters[3] + " = 1.0 / (REAL("+parameters[1]+") - (REAL(" + parameters[0] + ")) + 1.0)");
		result.add("END IF");

		return result;
	}

	public ArrayList<String> getGenCode(String[] parameters) throws IllegalParametersException
	{
        // Parameters are: m, n, result
        if (parameters.length != 3)
		{
			throw new IllegalParametersException("Trying to use discrete uniform distribution with illegal parameters.");
		}

        /*
        The code we want to generate looks like this:
        DO k=1, SIZE(result)
            result(k) = G05DYF(m, n)
        END DO
        */

        ArrayList<String> result = new ArrayList<String>();

        result.add("DO k=1, SIZE(" + parameters[2] + ")");
        result.add(parameters[2] + "(k) = G05DYF(" + parameters[0] + ", " + parameters[1] + ")");
        result.add("END DO");

        return result;

        /*
        Note:
        The code generated by this method uses the following already introduced variables:
        INTEGER :: k
        */
	}
}
