package PIANOS.datastructures;

import PIANOS.exceptions.*;

import java.util.*;

// import PIANOS.datastructures.Variable;

public abstract class Distribution
{
	int numberOfParameters;
	int[] intParameter;
	double[] realParameter;
	Variable[] variableParameter;
	int[] parameterType;
	String[] parameterString;


	public Distribution(int numberOfParameters)
	{
		this.numberOfParameters = numberOfParameters;
		intParameter = new int[numberOfParameters];
		realParameter = new double[numberOfParameters];
		variableParameter = new Variable[numberOfParameters];
		parameterType = new int[numberOfParameters];
		parameterString = new String[numberOfParameters];
	}

	public int getNumberOfParameters()
	{
		return numberOfParameters;
	}

	public boolean isInteger(int index) throws IndexOutOfBoundsException, NullPointerException
	{
		// Assumptions:
		// 1) This object is linked, that is, arrays
		// intParameter, realParameter, variableParameter
		// contain the necessary data and links

		if (index < 0 || index >= numberOfParameters)
		{
			throw new IndexOutOfBoundsException();
		}

		assert parameterType[index] <= 2;
        	assert parameterType[index] >= 0;

		switch (parameterType[index])
		{
			case 0: // integer
				return true;
			case 1: // double
				return false;
			case 2: // Variable
				// the "return"-line can generate a NullPointerException, so...
				if (variableParameter[index] == null)
				{
					throw new NullPointerException();
				}
				return variableParameter[index].isInteger();
		}
		// This should not be reached.
		assert false;

        return false;
	}

	public Object getParameter(int index)
	{
		// Assumptions:
		// 1) This object is linked, that is, arrays
		// intParameter, realParameter, variableParameter
		// contain the necessary data and links

		if (index < 0 || index >= numberOfParameters)
		{
			throw new IndexOutOfBoundsException();
		}

		assert parameterType[index] <= 2;
		assert parameterType[index] >= 0;

		switch (parameterType[index])
		{
			case 0: // integer
				return intParameter[index];
			case 1: // double
				return realParameter[index];
			case 2: // Variable
				// the "return"-line can generate a NullPointerException, so...
				if (variableParameter[index] == null)
				{
					throw new NullPointerException();
				}
				return variableParameter[index];
		}
		// This should not be reached.
		assert false;
        return null;
    }

    public void setParameter(int index, int parameter)
    {
        if (index < 0 || index >= numberOfParameters)
        {
            throw new IndexOutOfBoundsException();
        }

        parameterType[index]=0;
        intParameter[index]=parameter;
    }

    public void setParameter(int index, double parameter)
    {
        if (index < 0 || index >= numberOfParameters)
        {
            throw new IndexOutOfBoundsException();
        }

        parameterType[index]=1;
        realParameter[index]=parameter;
    }

    public void setParameter(int index, Variable parameter)
    {
        if (index < 0 || index >= numberOfParameters)
        {
            throw new IndexOutOfBoundsException();
        }

        parameterType[index]=2;
        variableParameter[index]=parameter;
    }

    public String getParameterString(int index)
    {
        if (index < 0 || index >= numberOfParameters)
        {
            throw new IndexOutOfBoundsException();
        }

        return parameterString[index];
    }

    public void setParameterString(int index, String parameter)
    {
        if (index < 0 || index >= numberOfParameters)
        {
            throw new IndexOutOfBoundsException();
        }

        parameterString[index]=parameter;
    }

    public String toString(){
	String toReturn = this.getClass().getName();
	if (toReturn.startsWith("PIANOS.datastructures."))
	    toReturn = toReturn.substring(22);
        if  (parameterString == null)
            return toReturn;
	toReturn += "("; 
        
	for (int i = 0; i < numberOfParameters; ++i){
	    toReturn += parameterString[i];
	    if (i + 1 != numberOfParameters)
		toReturn += ", ";
	}
	toReturn += ")";

	return toReturn;
    }

    public abstract ArrayList<String> getIntroduction() throws IllegalParametersException;

    public abstract ArrayList<String> getGenCode(String[] parameters) throws IllegalParametersException, MissingFunctionException;

    public abstract ArrayList<String> getFreqCode(String[] parameters) throws IllegalParametersException, MissingFunctionException;

}
