package PIANOS.datastructures;

import PIANOS.exceptions.*;

import java.util.*;

public class UserDefinedDistribution extends Distribution{

    private DistributionSkeleton skeleton;

    public UserDefinedDistribution (DistributionSkeleton skeleton){
	//Call the constructor of Distribution class
	super (skeleton.getNumberOfParameters());
	this.skeleton = skeleton;

	boolean[] paramType = skeleton.getTypeOfParameters();

	//Process the parameter types
	for (int i = 0; i < numberOfParameters; i++){
	    if (paramType[i] == true)
		parameterType[i] = 0;
	    else
		parameterType[i] = 1;
	}
    }

    public ArrayList<String> getIntroduction() throws IllegalParametersException{

		return new ArrayList<String>();

	}

    public ArrayList<String> getGenCode(String[] parameters) throws IllegalParametersException, MissingFunctionException{

	//Check that generating function exists
	if (!skeleton.hasGenFunction())
	    throw new MissingFunctionException ("User defined distribution " + skeleton.getName() + " does not have generator function.");

	//Check the number of parameters
	if (parameters.length != numberOfParameters + 1)
	    throw new IllegalParametersException ("User defined distribution " + skeleton.getName() + " has wrong number of parameters in the proposal distribution file." );
	
	//process the parameters
	String temp = parameters[0];
	for (int i = 1; i < numberOfParameters + 1; i++)
	    temp += ", " + parameters[i];


	//Create a result String
	ArrayList<String> result = new ArrayList<String>();
	result.add("CALL " + skeleton.getName() + "_gen(" + temp + ")");
	return result;

	/*
	String[] tester = new String[3];
	tester[0] = "This is a test String.";
	tester[1] = "User defined distribution: " + skeleton.getName();
	tester[2] = "Generator function, number of parameters: " + numberOfParameters;
	return tester;
	*/
    }

    public ArrayList<String> getFreqCode(String[] parameters) throws IllegalParametersException, MissingFunctionException{

	//Check that frequency function exists
	if (!skeleton.hasFreqFunction())
	    throw new MissingFunctionException ("User defined distribution " + skeleton.getName() + " does not have frequency function.");

	//Check the number of parameters
	if (parameters.length != numberOfParameters + 2)
	    throw new IllegalParametersException ("User defined distribution " + skeleton.getName() + " has wrong number of parameters in the model file");

	//process the parameters
	String temp = parameters[0];
	for (int i = 1; i < numberOfParameters + 2; i++)
	    temp += ", " + parameters[i];

	//Create a result String
	ArrayList<String> result = new ArrayList<String>();
	result.add("CALL " + skeleton.getName() + "_freq(" + temp + ")");

	return result;

	/*
	String[] tester = new String[3];
	tester[0] = "This is a test String.";
	tester[1] = "User defined distribution: " + skeleton.getName();
	tester[2] = "Frequency function, number of parameters: " + numberOfParameters;
	return tester;
	*/
    }

    public String toString(){
	String toReturn = skeleton.getName() +"(";
	for (int i = 0; i < numberOfParameters; ++i){
	    toReturn += parameterString[i];
	    if (i + 1 != numberOfParameters)
		toReturn += ", ";
	}
	toReturn += ")";

	return toReturn;
    }
    
}
