package PIANOS.generator;

import PIANOS.datastructures.ComputationalModel;
import PIANOS.io.*;
import java.io.*;
import java.util.*;

/**
 * This class is used to generate the Fortran module "definitions".
 */

public class Definitions {

    /** Generates and writes the Fortran module definitions.f90. 
     * @param  model The model this simulation is related to
     */
    public static void generateDefinitions(ComputationalModel model) throws IOException {
	FortranWriter out = new FortranWriter("definitions.f90");

	ArrayList <String> toWrite = new ArrayList<String>();

	toWrite.add("MODULE definitions");
	toWrite.add("");
	toWrite.add("TYPE variable_int");
	toWrite.add("");
	toWrite.add("LOGICAL :: is_data");
	toWrite.add("");
	toWrite.add("LOGICAL :: initial_value_set");
	toWrite.add("");
	toWrite.add("INTEGER :: value");
	toWrite.add("INTEGER :: new_value");
	toWrite.add("");
	toWrite.add("INTEGER :: successful_changes");
	toWrite.add("");
	if (model.getUpdateStrategy().equals("random")){
	    toWrite.add("! HARD-CODED: these two are here iff the update strategy is random");
	    toWrite.add("INTEGER :: updates_wanted");
	    toWrite.add("INTEGER :: update_count");
	}
	toWrite.add("");
	toWrite.add("END TYPE variable_int");
	toWrite.add("");
	toWrite.add("TYPE variable_real");
	toWrite.add("");
	toWrite.add("LOGICAL :: is_data");
	toWrite.add("");
	toWrite.add("LOGICAL :: initial_value_set");
	toWrite.add("");
	toWrite.add("REAL (KIND=SELECTED_REAL_KIND(8, 0)) :: value");
	toWrite.add("REAL (KIND=SELECTED_REAL_KIND(8, 0)) :: new_value");
	toWrite.add("");
	toWrite.add("INTEGER :: successful_changes");
	toWrite.add("");
	if (model.getUpdateStrategy().equals("random")){
	    toWrite.add("! HARD-CODED: these two are here iff the update strategy is random");
	    toWrite.add("INTEGER :: updates_wanted");
	    toWrite.add("INTEGER :: update_count");
	}
	toWrite.add("");
	toWrite.add("");
	toWrite.add("END TYPE variable_real");
	toWrite.add("");
	toWrite.add("");
	toWrite.add("TYPE variables_int");
	toWrite.add("");
	toWrite.add("INTEGER :: buffer_index");
	toWrite.add("INTEGER, ALLOCATABLE, DIMENSION(:) :: buffer");
	toWrite.add("");
	toWrite.add("! the places of the variables missing in data.");
	toWrite.add("INTEGER, ALLOCATABLE, DIMENSION(:) :: one_dim_missing");
	toWrite.add("INTEGER, ALLOCATABLE, DIMENSION(:, :) :: two_dim_missing");
	toWrite.add("! n, 2");
	toWrite.add("");
	toWrite.add("TYPE(variable_int), ALLOCATABLE, DIMENSION(:) :: one_dim");
	toWrite.add("TYPE(variable_int), ALLOCATABLE, DIMENSION(:, :) :: two_dim");
	toWrite.add("");
	toWrite.add("END TYPE variables_int");
	toWrite.add("");
	toWrite.add("TYPE variables_real");
	toWrite.add("");
	toWrite.add("INTEGER :: buffer_index");
	toWrite.add("REAL (KIND=SELECTED_REAL_KIND(8, 0)), ALLOCATABLE, DIMENSION(:) :: buffer");
	toWrite.add("");
	toWrite.add("! the places of the variables missing in data.");
	toWrite.add("INTEGER, ALLOCATABLE, DIMENSION(:) :: one_dim_missing");
	toWrite.add("INTEGER, ALLOCATABLE, DIMENSION(:, :) :: two_dim_missing");
	toWrite.add("");
	toWrite.add("");
	toWrite.add("TYPE(variable_real), ALLOCATABLE, DIMENSION(:) :: one_dim");
	toWrite.add("TYPE(variable_real), ALLOCATABLE, DIMENSION(:, :) :: two_dim");
	toWrite.add("");
	toWrite.add("END TYPE variables_real");
	toWrite.add("");
	toWrite.add("END MODULE definitions");

	out.write(toWrite.toArray(new String[toWrite.size()]));
    }
}
