package PIANOS.datastructures;

import PIANOS.exceptions.*;
import PIANOS.datastructures.*;

import java.util.*;

/*
Note: The thing calling generaterFreqCode and generateGenCode
must add _dp to real constants. For example, "1.0_dp", not "1.0".
*/

public class BinomialDistribution extends Distribution
{

    public BinomialDistribution()
    {
        super(2);
        parameterType[0] = 0; // integer
        parameterType[1] = 1; // real
    }

    public ArrayList<String>  getIntroduction()
    {
		ArrayList<String> result = new ArrayList<String>();

        result.add("INTEGER G05EYF");
        result.add("EXTERNAL G05EYF");
        result.add("EXTERNAL G05EDF");
        result.add("EXTERNAL G01BJF");

        return result;
    }

    public ArrayList<String> getFreqCode(String[] parameters) throws IllegalParametersException
    {
        // Parameters are: n, p, point, result
        if (parameters.length != 4)
		{
			throw new IllegalParametersException("Trying to use binomial distribution with illegal parameters.");
		}

        /*
        The code we want to generate looks like this:
        ifail=0
        CALL G01BJF(n, p, point, temp_real, temp_real, result, ifail)
        IF (ifail /= 0) THEN
            WRITE (*, *) 'Error when trying to calculate frequency of the binomial distribution, parameters: n = ', n, 'p = ', p
            STOP
        END IF
        */

        ArrayList<String> result = new ArrayList<String>();
        System.out.println("Using binomial distribution");
        result.add("ifail=0");
        result.add("CALL G01BJF(" + parameters[0] + ", " + parameters[1] + ", " +
            parameters[2] + ", temp_real, temp_real, " + parameters[3] + ", ifail)");
        result.add("IF (ifail /= 0) THEN");
        result.add("WRITE (*, *) 'Error when trying to calculate frequency of the binomial distribution, " +
            "parameters: n = '," + parameters[0] + ", 'p = '," + parameters[1]);
        result.add("STOP");
        result.add("END IF");
        return result;

        /*
        Note:
        The code generated by this method uses the following already introduced variables:
        INTEGER :: ifail
        REAL :: temp_real
        */

    }

    public ArrayList<String> getGenCode(String[] parameters) throws IllegalParametersException
    {
        // Parameters are: n, p, result
        if (parameters.length != 3)
		{
			throw new IllegalParametersException("Trying to use binomial distribution with illegal parameters.");
		}

        /*
        The code we want to generate looks like this:
        ifail=0
        CALL G05EDF(n, p, binomial_reference, SIZE(binomial_reference), ifail)
        IF (ifail /= 0) THEN
            WRITE (*, *) 'Error when trying to generate random numbers from binomial distribution, parameters: n = ', n, 'p = ', p
            STOP
        END IF
        DO k=0, SIZE(result)
            result(k)=G05EYF(binomial_reference, SIZE(binomial_reference))
        END DO
        */

        ArrayList<String> result = new ArrayList<String>();

        result.add("ifail = 0");
        result.add("ALLOCATE(binomial_reference(1 : NINT(20 + 20 * SQRT(" + parameters[0] + 
            " * " + parameters[1] + " * (1 - " + parameters[1] +")))))");
        result.add("CALL G05EDF(" + parameters[0] + ", " + parameters[1] +
            ", binomial_reference, SIZE(binomial_reference), ifail)");
        result.add("IF (ifail /= 0) THEN");
        result.add("WRITE (*, *) 'Error when trying to generate random numbers from binomial distribution, parameters: n = ', "
            + parameters[0] + ", ' p = ', " + parameters[1]);
        result.add("STOP");
        result.add("END IF");
        result.add("DO k=1, SIZE(" + parameters[2] +")");
        result.add(parameters[2] + "(k)=G05EYF(binomial_reference, SIZE(binomial_reference))");
        result.add("END DO");

        return result;

        /*
        Note:
        The code generated by this method uses the following already introduced variables:
        INTEGER :: k, ifail
        REAL(KIND=dp), DIMENSION(:), ALLOCATABLE :: binomial_reference

        NOTE: the dimension must be large enough!
        Optimum: NR = 20 + 20 * sqrt( N * P(1 -P))

        */
    }
}
