package PIANOS.datastructures;

import java.util.*;
import java.io.File;


public class ComputationalModel{
    private int iterations;
    private int burnIn;
    private int thinning;
    private String updateStrategy;
    private int maxSpatialNeighbours;
    private String modelFile;
    private String initialValueFile;
    private String outputFile;
    private String summaryFile;
    private String lastValuesFileName;

    private LinkedList<Variable> variableList;
    private LinkedList<Entity> entityList;
    private HashMap<String, Entity> entityMapper;
    private HashMap<String, Variable> variableMapper;
    
    private ArrayList<Variable> topologicalVariableList;
    
    public ComputationalModel(int iterations, int burnIn,
            int thinning, int maxSpatialNeighbours,
            String updateStrategy,
            LinkedList<Variable> variableList,
            LinkedList<Entity> entityList,
            HashMap<String, Entity> entityMapper,
            HashMap<String, Variable> variableMapper,
            String modelFile,
            String initialValueFile, String outputFile, String summaryFile){
        this.iterations = iterations;
        this.burnIn = burnIn;
        this.thinning = thinning;
        this.maxSpatialNeighbours = maxSpatialNeighbours;
        this.updateStrategy = updateStrategy;
        this.variableList = variableList;
        this.entityList = entityList;
        this.entityMapper = entityMapper;
        this.variableMapper = variableMapper;
        this.modelFile = modelFile;
        this.initialValueFile = initialValueFile;
        this.outputFile = outputFile;
        this.summaryFile = summaryFile;
        
        topologicalVariableList = new ArrayList<Variable>();
        
        topologicalVariableList.addAll(variableList);
        
        for (Entity entity : entityList) {
            topologicalVariableList.addAll(entity.getVariableList());
        }
        
    }
    
    public int getIterations(){
        return iterations;
    }
    
    public int getBurnIn(){
        return burnIn;
    }
    
    public int getThinning(){
        return thinning;
    }
    
    public int getNeighbourCount(){
        return this.maxSpatialNeighbours;
    }
    
    public String getUpdateStrategy(){
        return updateStrategy;
    }
    
    public LinkedList<Variable> getVariableList(){
        return variableList;
    }
    public LinkedList<Entity> getEntityList(){
        return entityList;
    }
    
    public HashMap<String, Variable> getVariableMapper(){
        return variableMapper;
    }
    public HashMap<String, Entity> getEntityMapper(){
        return entityMapper;
    }
    
    public String getModelFileName(){ // use this
        return modelFile;
    }

    public String getModelFile(){ // not this
        return modelFile;
    }

    public String getOutputFileName(){ // use this
        return this.outputFile;
    }

    public String getOutputFile(){ // not this
        return this.outputFile;
    }

    public String getInitialValueFile(){
        return this.initialValueFile;
    }

    public String getInitialValueFileName(){ // use this
        return this.initialValueFile;
    }
    
    public String getSummaryFile(){ // not this
        return this.summaryFile;
    }

    public String getSummaryFileName(){ // use this
        return this.summaryFile;
    }

    public void setLastValuesFileName(String name){ 
        lastValuesFileName = name;
    }

    public String getLastValuesFileName(){
        return this.lastValuesFileName;
    }
    
    public ArrayList<Variable> getTopologicalVariableList() {
        return topologicalVariableList;
    }
    
    public String toString(){
        String toReturn = "";
        
        toReturn += "Model summary:" + "\n";
        toReturn += "File name: " + modelFile + "\n";
        toReturn += "Iterations: " + iterations + "\n";
        toReturn += "Burn-in: " + burnIn + "\n";
        toReturn += "Thinning: " + thinning + "\n";
        toReturn += "Update strategy: " + updateStrategy + "\n";
        toReturn += "Structure: " + "\n";
        toReturn += "\tGlobal variables: " + "\n";
        
        for (Variable var : variableList){
            toReturn += "\t" + var + "\n";
        }
        
        for (Entity e : entityList){
            toReturn += e + "\n";
        }
        return toReturn;
    }
}
