package PIANOS.datastructures;

import PIANOS.exceptions.*;
import java.util.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.StringTokenizer;


public class DistributionFactory{

    private HashMap<String, DistributionSkeleton> userDistributions;



    public DistributionFactory (String name) throws MissingFunctionException, SyntaxException{

		userDistributions = new HashMap<String, DistributionSkeleton>();

		Scanner reader = null;
		try{
		    reader = new Scanner (new File (name));
		}catch (FileNotFoundException e){
		    throw new MissingFunctionException ("The user defined function file could not be found!");
		    // The file was not found
		}

		StringTokenizer tokenizer;
		String temp;
		int count, index, line = 0, userParameters = 0;

		String nameOfFunction;
		boolean[] type;
		boolean hasGen;

		DistributionSkeleton skeleton;

		// Do as long as the first character in the line is '!'
		while (reader.hasNextLine()){
		    temp = reader.nextLine();

		    if (temp.length() > 0 && temp.charAt(0) != '!')
			break;

		    line++;

		    if (temp.length() == 0)
			continue;

		    //Create a new StringTokenizer, tokens are sparated with '!', ' ' or ','
		    tokenizer = new StringTokenizer (temp, " !");
		    count = tokenizer.countTokens();

		    //Every line has to have at least 3 tokes: name, number of parameters and boolean value
		    count = count - 3;
		    if (count < 0)
			//Line has less than 3 tokens...
				throw new SyntaxException ("Syntax error has been detected in user defined function file \"" + name + "\"\nLine " + line + " does not have enough arguments.");

		    //First token, the name of the function, is read
		    nameOfFunction = tokenizer.nextToken();
		    nameOfFunction = nameOfFunction.toLowerCase();
		    if (!nameOfFunction.startsWith("user_"))
			throw new SyntaxException ("Syntax error has been detected in user defined function file \"" + name + "\""
						   + "\n Line " + line + " has an incorrect function name. The function name needs to start with \"user_\"");

		    //Second token, the number of parameters, is compared to total number of tokens
		    try{
			userParameters = Integer.parseInt(tokenizer.nextToken());
		    }catch (NumberFormatException e){
			throw new SyntaxException ("Syntax error has been detected in user defined function file \"" + name + "\", line " + line
						   + "\nNumber of parameters field is not an integer.");
		    }

		    if (count != userParameters)
			//Oops... number of parameters and calculated number of parameters vary from eachother!
				throw new SyntaxException ("Syntax error has been detected in user defined function file \"" + name +
						   "\"\nNumber of arguments does not corespond to number of parameters in line" + line);

		    type = new boolean [count];
		    index = 0;

		    //Parameter types...
		    while (index < count){
				temp = tokenizer.nextToken();
				temp = temp.toLowerCase();
				if (temp.equals("integer"))
			 	   type[index] = true;
				else if (temp.equals("real"))
				    type[index] = false;
				else
			 	   //Syntax error in parameter type
				    throw new SyntaxException ("Syntax error has been detected in user defined function file \"" + name +
						       "\"\nLine " + line + " has an invalid parameter type.");
				index++;
	   		}

		    //Last boolean value...
		    temp = tokenizer.nextToken();
		    temp = temp.toLowerCase();
		    if (temp.equals(".true."))
				hasGen = true;
		    else if (temp.equals(".false."))
				hasGen = false;
		    else
				//Syntax error...
				throw new SyntaxException ("Syntax error has been detected in user defined function file \"" + name +
							   "\"\nLine " + line + " has an invalid boolean value.");

		    //Everything is fine, new skeleton is created
		    skeleton = new DistributionSkeleton (nameOfFunction, count, type, true, hasGen);
		    DistributionSkeleton check = null;
		    check = userDistributions.get(nameOfFunction);
		    if(check != null)
			throw new SyntaxException ("User defined distribution '" + nameOfFunction + "' has been defined twice in the user defined distribution file!"
						   + "\nPlease remove the second definition.");
		    userDistributions.put(nameOfFunction, skeleton);
		}
    }

	public Distribution getDistribution (String name) throws MissingDistributionException{

		name = name.toLowerCase();

		if (name.equals("discrete_uniform"))
		{
		return new DiscreteUniformDistribution();
		}
		else if (name.equals("binomial"))
		{
		return new BinomialDistribution();
		}
		else if (name.equals("poisson"))
		{
		return new PoissonDistribution();
		}
		else if (name.equals("continuous_uniform"))
		{
		return new ContinuousUniformDistribution();
		}
		else if (name.equals("beta"))
		{
		return new BetaDistribution();
		}

		DistributionSkeleton skeleton = userDistributions.get(name);
		if (skeleton == null)
			throw new MissingDistributionException ("Could not find user-defined distribution called \"" + name + "\".");

		return new UserDefinedDistribution (skeleton);

	}


}
